/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.rxjava.ext.web.handler;

import java.util.Map;
import rx.Observable;
import rx.Single;
import io.vertx.rxjava.ext.web.templ.TemplateEngine;
import io.vertx.rxjava.ext.web.RoutingContext;
import io.vertx.core.Handler;

/**
 *
 * A handler which renders responses using a template engine and where the template name is selected from the URI
 * path.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.web.handler.TemplateHandler original} non RX-ified interface using Vert.x codegen.
 */

@io.vertx.lang.rxjava.RxGen(io.vertx.ext.web.handler.TemplateHandler.class)
public class TemplateHandler implements io.vertx.core.Handler<RoutingContext> {

  public static final io.vertx.lang.rxjava.TypeArg<TemplateHandler> __TYPE_ARG = new io.vertx.lang.rxjava.TypeArg<>(
    obj -> new TemplateHandler((io.vertx.ext.web.handler.TemplateHandler) obj),
    TemplateHandler::getDelegate
  );

  private final io.vertx.ext.web.handler.TemplateHandler delegate;
  
  public TemplateHandler(io.vertx.ext.web.handler.TemplateHandler delegate) {
    this.delegate = delegate;
  }

  public io.vertx.ext.web.handler.TemplateHandler getDelegate() {
    return delegate;
  }

  public void handle(RoutingContext arg0) { 
    delegate.handle(arg0.getDelegate());
  }

  /**
   * Set the index template
   * @param indexTemplate the index template
   * @return a reference to this, so the API can be used fluently
   */
  public TemplateHandler setIndexTemplate(String indexTemplate) { 
    delegate.setIndexTemplate(indexTemplate);
    return this;
  }

  /**
   * Create a handler
   * @param engine the template engine
   * @return the handler
   */
  public static TemplateHandler create(TemplateEngine engine) { 
    TemplateHandler ret = TemplateHandler.newInstance(io.vertx.ext.web.handler.TemplateHandler.create(engine.getDelegate()));
    return ret;
  }

  /**
   * Create a handler
   * @param engine the template engine
   * @param templateDirectory the template directory where templates will be looked for
   * @param contentType the content type header to be used in the response
   * @return the handler
   */
  public static TemplateHandler create(TemplateEngine engine, String templateDirectory, String contentType) { 
    TemplateHandler ret = TemplateHandler.newInstance(io.vertx.ext.web.handler.TemplateHandler.create(engine.getDelegate(), templateDirectory, contentType));
    return ret;
  }


  public static  TemplateHandler newInstance(io.vertx.ext.web.handler.TemplateHandler arg) {
    return arg != null ? new TemplateHandler(arg) : null;
  }
}
